package quimufu.colourful_portals;

import net.minecraft.block.*;
import net.minecraft.component.DataComponentTypes;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.fluid.Fluid;
import net.minecraft.fluid.FluidState;
import net.minecraft.item.*;
import net.minecraft.item.tooltip.TooltipType;
import net.minecraft.network.packet.s2c.play.ParticleS2CPacket;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.server.world.ServerWorld;
import net.minecraft.state.StateManager;
import net.minecraft.state.property.EnumProperty;
import net.minecraft.text.Text;
import net.minecraft.util.DyeColor;
import net.minecraft.util.Hand;
import net.minecraft.util.ItemActionResult;
import net.minecraft.util.hit.BlockHitResult;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.random.Random;
import net.minecraft.util.shape.VoxelShape;
import net.minecraft.util.shape.VoxelShapes;
import net.minecraft.world.BlockView;
import net.minecraft.world.World;
import net.minecraft.world.WorldAccess;
import net.minecraft.world.WorldView;
import org.jetbrains.annotations.Nullable;

import java.util.List;

public class ColourfulAirBlock extends Block implements FluidFillable {
    public static final EnumProperty<DyeColor> DYE_COLOR = EnumProperty.of("colour", DyeColor.class);


    public ColourfulAirBlock(Settings settings) {
        super(settings);
        this.setDefaultState(this.stateManager.getDefaultState().with(DYE_COLOR, DyeColor.BLACK));
    }

    @Override
    protected ItemActionResult onUseWithItem(ItemStack stack, BlockState state, World world, BlockPos pos, PlayerEntity player, Hand hand, BlockHitResult hit) {
        if(stack.getItem() == Items.GLASS_BOTTLE){
            ItemStack filledBottle = getPickStack(world, pos, state);
            player.setStackInHand(hand, ItemUsage.exchangeStack(stack, player, filledBottle));
            if(!world.isClient){
                world.setBlockState(pos, Blocks.AIR.getDefaultState());
            }
            return ItemActionResult.success(world.isClient);
        }
        return ItemActionResult.PASS_TO_DEFAULT_BLOCK_INTERACTION;
    }

    @Override
    protected void onBlockAdded(BlockState state, World world, BlockPos pos, BlockState oldState, boolean notify) {
        scheduleTick(world, pos, true);
    }


    @Override
    public void appendTooltip(ItemStack stack,
                              Item.TooltipContext context,
                              List<Text> tooltip, TooltipType options) {

        DyeColor color = stack.getOrDefault(DataComponentTypes.BASE_COLOR, DyeColor.BLACK);
        tooltip.add(Text.translatable("color.minecraft." + color.getName())
                    .withColor(color.getEntityColor()));

    }

    @Override
    public ItemStack getPickStack(WorldView world, BlockPos pos, BlockState state) {
        ItemStack stack = ColourfulPortalsMod.COLOURFUL_AIR_BOTTLE_ITEM.getDefaultStack();
        stack.set(DataComponentTypes.BASE_COLOR, state.get(DYE_COLOR));
        return stack;
    }

    @Override
    public VoxelShape getOutlineShape(BlockState state, BlockView world, BlockPos blockPos, ShapeContext shapeContext) {
        if (shapeContext.isHolding(Items.DEBUG_STICK)
            || shapeContext.isHolding(ColourfulPortalsMod.COLOURFUL_AIR_BOTTLE_ITEM)
            || shapeContext.isHolding(Items.GLASS_BOTTLE)) {
            return VoxelShapes.fullCube();
        }
        return VoxelShapes.empty();
    }

    @Override
    protected BlockRenderType getRenderType(BlockState state) {
        return BlockRenderType.INVISIBLE;
    }

    @Override
    protected void randomTick(BlockState state, ServerWorld world, BlockPos pos, Random random) {
        scheduleTick(world, pos, false);
    }

    @Override
    protected void scheduledTick(BlockState state, ServerWorld world, BlockPos pos, Random random) {
        DyeColor dyeColor = state.get(DYE_COLOR);
        for (ServerPlayerEntity player : world.getPlayers()) {
            double x = pos.getX() + .5D;
            double y = pos.getY() + .5D;
            double z = pos.getZ() + .5D;
            ParticleS2CPacket particleS2CPacket = new ParticleS2CPacket(
                    ColourfulPortalsMod.COLOURFUL_AIR_PARTICLE_BY_COLOUR.get(dyeColor),
                    true,
                    x, y, z,
                    0.25F, 0.25F, 0.25F,
                    0,
                    1);

            world.sendToPlayerIfNearby(player, true, x, y, z, particleS2CPacket);
        }
        scheduleTick(world, pos, false);
    }

    private void scheduleTick(WorldAccess world, BlockPos pos, boolean asap) {
        if (!world.isClient() && !world.getBlockTickScheduler().isQueued(pos, this)) {
            if(asap){
                world.scheduleBlockTick(pos, this, 2);
            } else if (world.getClosestPlayer(pos.getX(),pos.getY(),pos.getZ(),512,false) != null) {
                world.scheduleBlockTick(pos, this, 20);
            }
        }
    }

    @Override
    public BlockState getPlacementState(ItemPlacementContext ctx) {
        DyeColor color = ctx.getStack().getOrDefault(DataComponentTypes.BASE_COLOR, DyeColor.BLACK);
        return this.getDefaultState().with(DYE_COLOR, color);
    }

    public BlockState getRandomState(Random random) {
        DyeColor color = DyeColor.values()[random.nextInt(DyeColor.values().length)];
        return this.getDefaultState().with(DYE_COLOR, color);
    }

    @Override
    protected void appendProperties(StateManager.Builder<Block, BlockState> builder) {
        builder.add(DYE_COLOR);
    }

    @Override
    public boolean canFillWithFluid(@Nullable PlayerEntity player, BlockView world, BlockPos pos, BlockState state, Fluid fluid) {
        return false;
    }

    @Override
    public boolean tryFillWithFluid(WorldAccess world, BlockPos pos, BlockState state, FluidState fluidState) {
        return false;
    }
}
