package quimufu.colourful_portals.client.rendering.fluid;

import net.fabricmc.fabric.api.client.render.fluid.v1.FluidRenderHandler;
import net.fabricmc.fabric.api.client.render.fluid.v1.FluidRenderHandlerRegistry;
import net.minecraft.block.BlockState;
import net.minecraft.client.texture.Sprite;
import net.minecraft.fluid.FluidState;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Direction;
import net.minecraft.world.BlockRenderView;
import org.joml.Quaternionf;
import org.joml.Vector3f;
import quimufu.colourful_portals.ColourfulPortalsMod;
import quimufu.colourful_portals.portal_fluid.NullableAxis;
import quimufu.colourful_portals.portal_fluid.PortalFluid;

import java.util.Arrays;
import java.util.EnumMap;
import java.util.Map;

import static quimufu.colourful_portals.ColourfulPortalsMod.LOGGER;

public class CommonPortalFluidRendererV2 implements CommonPortalFluidRenderer {

    private Sprite sprite = null;

    private final float[] heights = new float[9];
    private final int[] heightsC = new int[9];

    private static final EnumMap<NullableAxis, Quaternionf> quaternions =
            new EnumMap<>(Map.of(
                    NullableAxis.X, new Quaternionf()
                            .rotationTo(0,1,0,-1,0,0),
                    NullableAxis.Y, new Quaternionf(),//identity
                    NullableAxis.Z, new Quaternionf()
                            .rotationTo(0,1,0,0,0,-1),
                    NullableAxis.NULL, new Quaternionf()//identity
            ));

    private final int[] amounts = new int[9];

    private final float[] quads = new float[]{
            //Up
            -0.5f, 0.5f, 0.0f,
            0.0f, 0.5f, 0.0f,
            0.0f, 0.5f, -0.5f,
            -0.5f, 0.5f, -0.5f,

            0.0f, 0.5f, 0.0f,
            0.5f, 0.5f, 0.0f,
            0.5f, 0.5f, -0.5f,
            0.0f, 0.5f, -0.5f,

            -0.5f, 0.5f, 0.5f,
            0.0f, 0.5f, 0.5f,
            0.0f, 0.5f, 0.0f,
            -0.5f, 0.5f, 0.0f,

            0.0f, 0.5f, 0.5f,
            0.5f, 0.5f, 0.5f,
            0.5f, 0.5f, 0.0f,
            0.0f, 0.5f, 0.0f,

            //Down
            -0.5f, -0.5f, -0.5f,
            0.0f, -0.5f, -0.5f,
            0.0f, -0.5f, 0.0f,
            -0.5f, -0.5f, 0.0f,

            0.0f, -0.5f, -0.5f,
            0.5f, -0.5f, -0.5f,
            0.5f, -0.5f, 0.0f,
            0.0f, -0.5f, 0.0f,

            -0.5f, -0.5f, 0.0f,
            0.0f, -0.5f, 0.0f,
            0.0f, -0.5f, 0.5f,
            -0.5f, -0.5f, 0.5f,

            0.0f, -0.5f, 0.0f,
            0.5f, -0.5f, 0.0f,
            0.5f, -0.5f, 0.5f,
            0.0f, -0.5f, 0.5f,

            //NORTH
            -0.5f, 0.5f, -0.5f,
            0.0f, 0.5f, -0.5f,
            0.0f, -0.5f, -0.5f,
            -0.5f, -0.5f, -0.5f,

            0.0f, 0.5f, -0.5f,
            0.5f, 0.5f, -0.5f,
            0.5f, -0.5f, -0.5f,
            0.0f, -0.5f, -0.5f,

            //SOUTH
            -0.5f, -0.5f, 0.5f,
            0.0f, -0.5f, 0.5f,
            0.0f, 0.5f, 0.5f,
            -0.5f, 0.5f, 0.5f,

            0.0f, -0.5f, 0.5f,
            0.5f, -0.5f, 0.5f,
            0.5f, 0.5f, 0.5f,
            0.0f, 0.5f, 0.5f,

            //WEST
            -0.5f, -0.5f, -0.5f,
            -0.5f, -0.5f, 0.0f,
            -0.5f, 0.5f, 0.0f,
            -0.5f, 0.5f, -0.5f,

            -0.5f, -0.5f, 0.0f,
            -0.5f, -0.5f, 0.5f,
            -0.5f, 0.5f, 0.5f,
            -0.5f, 0.5f, 0.0f,

            //EAST
            0.5f, 0.5f, -0.5f,
            0.5f, 0.5f, 0.0f,
            0.5f, -0.5f, 0.0f,
            0.5f, -0.5f, -0.5f,

            0.5f, 0.5f, 0.0f,
            0.5f, 0.5f, 0.5f,
            0.5f, -0.5f, 0.5f,
            0.5f, -0.5f, 0.0f,
    };

    public boolean render(BlockRenderView world, FluidState fluidState, BlockPos pos, Vector3f offset, VertexEater vertexEater) {

        FluidRenderHandler handler = FluidRenderHandlerRegistry.INSTANCE.get(fluidState.getFluid());
        sprite = handler.getFluidSprites(null, null, fluidState)[0];

        NullableAxis axis = fluidState.get(PortalFluid.AXIS);

        int amount = fluidState.get(PortalFluid.AMOUNT);
        if (amount == 0)
            return false;

        //get neighbours heights:
        saveAmounts(world, pos, axis);

        //offsets of corners:
        Arrays.fill(heights, 0);
        Arrays.fill(heightsC, 0);
        for (int a = 0; a < 3; a++) {
            for (int b = 0; b < 3; b++) {
                if (amounts[a * 3 + b] <= -1) {
                    if ((a == 1) ^ (b == 1)) {
                        heights[a * 3 + b] += amounts[4] / 32f;
                        heightsC[a * 3 + b]++;
                    }
                    continue;
                }
                if (a < 2 && b < 2) {
                    heights[0] += amounts[a * 3 + b] / 32f;
                    heightsC[0]++;
                }
                if (a < 2 && b > 0) {
                    heights[2] += amounts[a * 3 + b] / 32f;
                    heightsC[2]++;
                }
                if (a > 0 && b < 2) {
                    heights[6] += amounts[a * 3 + b] / 32f;
                    heightsC[6]++;
                }
                if (a > 0 && b > 0) {
                    heights[8] += amounts[a * 3 + b] / 32f;
                    heightsC[8]++;
                }
                if ((a == 1) ^ (b == 1)) {
                    heights[a * 3 + b] += amounts[a * 3 + b] / 32f;
                    heights[a * 3 + b] += amounts[4] / 32f; //always != -1
                    heightsC[a * 3 + b] += 2;
                }
                if ((a == 1) && (b == 1)) {
                    heights[4] = amounts[4] / 32f;//always != -1
                    heightsC[4]++;
                }
            }
        }
        LOGGER.info("{} {} {}\n{} {} {}\n{} {} {}", amounts[0], amounts[1], amounts[2], amounts[3], amounts[4], amounts[5], amounts[6], amounts[7], amounts[8]);

        for (int a = 0; a < 9; a++) {
            heights[a] = heights[a] / heightsC[a];
        }
        LOGGER.info("{} {} {}\n{} {} {}\n{} {} {}", heights[0], heights[1], heights[2], heights[3], heights[4], heights[5], heights[6], heights[7], heights[8]);
        for (int i = 0; i < quads.length / 3; i++) {
            int x = (int) ((quads[i * 3] + 0.5f) * 2);
            int z = (int) ((quads[i * 3 + 2] + 0.5f) * 2);
            if (quads[i * 3 + 1] > 0f) {
                quads[i * 3 + 1] = 0f + heights[x * 3 + z];
            } else {
                quads[i * 3 + 1] = 0f - heights[x * 3 + z];
            }
        }
        vertexEater.setSprite(sprite);
        int endUp = 4;
        int endDown = 8;
        int endNorth = 10;
        int endSouth = 12;
        int endWest = 14;
        int endEast = 16;
        for (Direction value : Direction.values()) {
            switch (value) {
                case UP -> {
                    for (int i = 0; i < endUp; i++) {
                        for (int j = 0; j < 4; j++) {

                            vertexEater.eatVertex(
                                    quads[i * 12 + j * 3],
                                    quads[i * 12 + j * 3 + 1],
                                    quads[i * 12 + j * 3 + 2],
                                    sprite.getFrameU(quads[i * 12 + j * 3] + 0.5f),
                                    sprite.getFrameV(quads[i * 12 + j * 3 + 2] + 0.5f));
                        }
                        vertexEater.drawQuad(offset, quaternions.get(axis), Direction.UP);
                    }
                }
                case DOWN -> {
                    for (int i = endUp; i < endDown; i++) {
                        for (int j = 0; j < 4; j++) {
                            vertexEater.eatVertex(
                                    quads[i * 12 + j * 3],
                                    quads[i * 12 + j * 3 + 1],
                                    quads[i * 12 + j * 3 + 2],
                                    sprite.getFrameU(quads[i * 12 + j * 3] + 0.5f),
                                    sprite.getFrameV(quads[i * 12 + j * 3 + 2] + 0.5f));
                        }
                        vertexEater.drawQuad(offset, quaternions.get(axis), Direction.DOWN);
                    }
                }
                case NORTH -> {
                    if (amounts[3] >= -1) {
                        continue;
                    }
                    for (int i = endDown; i < endNorth; i++) {
                        for (int j = 0; j < 4; j++) {
                            vertexEater.eatVertex(
                                    quads[i * 12 + j * 3],
                                    quads[i * 12 + j * 3 + 1],
                                    quads[i * 12 + j * 3 + 2],
                                    sprite.getFrameU(quads[i * 12 + j * 3] + 0.5f),
                                    sprite.getFrameV(quads[i * 12 + j * 3 + 1] + 0.5f));
                        }
                        vertexEater.drawQuad(offset, quaternions.get(axis), Direction.NORTH);
                    }
                }
                case SOUTH -> {
                    if (amounts[5] >= -1) {
                        continue;
                    }
                    for (int i = endNorth; i < endSouth; i++) {
                        for (int j = 0; j < 4; j++) {
                            vertexEater.eatVertex(
                                    quads[i * 12 + j * 3],
                                    quads[i * 12 + j * 3 + 1],
                                    quads[i * 12 + j * 3 + 2],
                                    sprite.getFrameU(quads[i * 12 + j * 3] + 0.5f),
                                    sprite.getFrameV(quads[i * 12 + j * 3 + 1] + 0.5f));
                        }
                        vertexEater.drawQuad(offset, quaternions.get(axis), Direction.SOUTH);
                    }
                }
                case WEST -> {
                    if (amounts[1] >= -1) {
                        continue;
                    }
                    for (int i = endSouth; i < endWest; i++) {
                        for (int j = 0; j < 4; j++) {
                            vertexEater.eatVertex(
                                    quads[i * 12 + j * 3],
                                    quads[i * 12 + j * 3 + 1],
                                    quads[i * 12 + j * 3 + 2],
                                    sprite.getFrameU(quads[i * 12 + j * 3 + 1] + 0.5f),
                                    sprite.getFrameV(quads[i * 12 + j * 3 + 2] + 0.5f));
                        }
                        vertexEater.drawQuad(offset, quaternions.get(axis), Direction.WEST);
                    }
                }
                case EAST -> {
                    if (amounts[7] >= -1) {
                        continue;
                    }
                    for (int i = endWest; i < endEast; i++) {
                        for (int j = 0; j < 4; j++) {
                            vertexEater.eatVertex(
                                    quads[i * 12 + j * 3],
                                    quads[i * 12 + j * 3 + 1],
                                    quads[i * 12 + j * 3 + 2],
                                    sprite.getFrameU(quads[i * 12 + j * 3 + 1] + 0.5f),
                                    sprite.getFrameV(quads[i * 12 + j * 3 + 2] + 0.5f));
                        }
                        vertexEater.drawQuad(offset, quaternions.get(axis), Direction.EAST);
                    }
                }
            }
        }
        return true;
    }

    private void saveAmounts(BlockRenderView world, BlockPos pos, NullableAxis axis) {
        BlockPos.Mutable curr = new BlockPos.Mutable();
        switch (axis) {
            case NULL -> {
                Arrays.fill(amounts, -2);
                amounts[4] = 16;
            }
            case X -> {
                for (int y = 0; y < 3; y++) {
                    for (int z = 0; z < 3; z++) {
                        saveAmount(world, pos, curr, 0, y - 1, z - 1, y, z);
                    }
                }
            }
            case Y -> {
                for (int x = 0; x < 3; x++) {
                    for (int z = 0; z < 3; z++) {
                        saveAmount(world, pos, curr, x - 1, 0, z - 1, x, z);
                    }
                }
            }
            case Z -> {
                for (int x = 0; x < 3; x++) {
                    for (int y = 0; y < 3; y++) {
                        saveAmount(world, pos, curr, x - 1, y - 1, 0, x, y);
                    }
                }
            }
        }
    }

    private void saveAmount(BlockRenderView world, BlockPos pos, BlockPos.Mutable curr, int dx, int dy, int dz, int major, int minor) {
        curr.set(pos);
        curr.move(dx, dy, dz);
        BlockState blockState = world.getBlockState(curr);
        FluidState fluidState = blockState
                .getFluidState();
        if (fluidState.isOf(ColourfulPortalsMod.PORTAL_FLUID)) {
            amounts[major * 3 + minor] = fluidState.get(PortalFluid.AMOUNT);
        } else {
            Direction direction = Direction.fromVector(dx, dy, dz);
            if (direction != null && blockState.isSideSolidFullSquare(world, curr, direction.getOpposite())) {
                amounts[major * 3 + minor] = -1;
            } else {
                amounts[major * 3 + minor] = -2;
            }
        }
    }

}
