package quimufu.colourful_portals.client.rendering.fluid;

import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.fabricmc.fabric.api.client.render.fluid.v1.FluidRenderHandler;
import net.minecraft.block.BlockState;
import net.minecraft.client.render.VertexConsumer;
import net.minecraft.client.texture.Sprite;
import net.minecraft.client.texture.SpriteAtlasTexture;
import net.minecraft.fluid.FluidState;
import net.minecraft.util.Identifier;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Direction;
import net.minecraft.world.BlockRenderView;
import org.jetbrains.annotations.Nullable;
import org.joml.Quaternionf;
import org.joml.Vector3f;
import quimufu.colourful_portals.ColourfulPortalsMod;
import quimufu.colourful_portals.client.ColourfulPortalsModClient;

@Environment(value = EnvType.CLIENT)
public class PortalFluidRenderHandler implements FluidRenderHandler, VertexEater {
    Identifier PORTAL_FLUID_STILL = Identifier.of(ColourfulPortalsMod.MOD_ID, "block/portal_still");

    private Sprite sprite = null;
    private final ThreadLocal<VertexConsumer> vertexConsumer = new ThreadLocal<>();
    private final ThreadLocal<Integer> i = new ThreadLocal<>();

    private final ThreadLocal<float[][]> vertices = new ThreadLocal<>();
    private final ThreadLocal<Vector3f> tmpVector = ThreadLocal.withInitial(Vector3f::new);

    @Override
    public Sprite[] getFluidSprites(@Nullable BlockRenderView view, @Nullable BlockPos pos, FluidState state) {
        return new Sprite[]{sprite};
    }

    @Override
    public int getFluidColor(@Nullable BlockRenderView view, @Nullable BlockPos pos, FluidState state) {
        return 0;
    }

    @Override
    public void renderFluid(BlockPos pos, BlockRenderView world, VertexConsumer vertexConsumer, BlockState blockState, FluidState fluidState) {
        this.vertexConsumer.set(vertexConsumer);
        Vector3f offset = new Vector3f(pos.getX() & 0xF, pos.getY() & 0xF, pos.getZ() & 0xF);
        offset.add(0.5f,0.5f,0.5f);
        if (vertices.get() == null) {
            vertices.set(new float[4][5]);
        }
        ColourfulPortalsModClient.FLUID_RENDERER.get()
                .render(world, fluidState, pos, offset, this);
    }


    @Override
    public void reloadTextures(SpriteAtlasTexture textureAtlas) {
        sprite = textureAtlas.getSprite(PORTAL_FLUID_STILL);

    }

    @Override
    public void setSprite(Sprite sprite) {

    }

    @Override
    public void eatVertex(float x, float y, float z, float u, float v) {
        float[][] floats = vertices.get();
        int idx = i.get() == null ? 0 : i.get();
        floats[idx][0] = x;
        floats[idx][1] = y;
        floats[idx][2] = z;
        floats[idx][3] = u;
        floats[idx][4] = v;
        i.set((idx + 1) % 4);
    }

    @Override
    public void drawQuad(Vector3f offset, Quaternionf rotation, Direction direction) {
        float offX = offset.x;
        float offY = offset.y;
        float offZ = offset.z;
        Vector3f f = tmpVector.get();
        for (float[] v : vertices.get()) {
            f.set(v);
            rotation.transform(f);
            vertexConsumer.get()
                    .vertex(offX + f.x, offY + f.y, offZ + f.z)
                    .color(1f, 1f, 1f, 1.0f)
                    .texture(v[3], v[4])
                    .light(16)
                    .normal(0.0f, 1.0f, 0.0f);

        }


    }
}
