package quimufu.colourful_portals.client.rendering.fluid;

import me.jellysquid.mods.sodium.client.model.quad.ModelQuad;
import me.jellysquid.mods.sodium.client.model.quad.ModelQuadView;
import me.jellysquid.mods.sodium.client.model.quad.ModelQuadViewMutable;
import me.jellysquid.mods.sodium.client.model.quad.properties.ModelQuadFacing;
import me.jellysquid.mods.sodium.client.render.chunk.compile.ChunkBuildBuffers;
import me.jellysquid.mods.sodium.client.render.chunk.compile.buffers.ChunkModelBuilder;
import me.jellysquid.mods.sodium.client.render.chunk.terrain.material.DefaultMaterials;
import me.jellysquid.mods.sodium.client.render.chunk.terrain.material.Material;
import me.jellysquid.mods.sodium.client.render.chunk.vertex.format.ChunkVertexEncoder;
import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.minecraft.client.texture.Sprite;
import net.minecraft.fluid.FluidState;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Direction;
import net.minecraft.world.BlockRenderView;
import org.joml.Quaternionf;
import org.joml.Vector3f;
import quimufu.colourful_portals.client.ColourfulPortalsModClient;

@Environment(value = EnvType.CLIENT)
public class SodiumPortalFluidRenderHandler implements VertexEater {

    private Sprite sprite = null;
    private final ModelQuadViewMutable quad = new ModelQuad();

    private final CommonPortalFluidRenderer commonPortalFluidRenderer = ColourfulPortalsModClient.FLUID_RENDERER.get();
    private final ChunkVertexEncoder.Vertex[] vertices = ChunkVertexEncoder.Vertex.uninitializedQuad();
    private int i;
    private ChunkModelBuilder chunkModelBuilder;
    private Material material;

    public SodiumPortalFluidRenderHandler() {
    }

    public boolean render(BlockRenderView world, FluidState fluidState, BlockPos pos, BlockPos offset, ChunkBuildBuffers buffers) {
        material = DefaultMaterials.forFluidState(fluidState);
        this.chunkModelBuilder = buffers.get(material);
        Vector3f vOffset = new Vector3f(offset.getX(), offset.getY(), offset.getZ());
        vOffset.add(0.5f,0.5f,0.5f);
        return commonPortalFluidRenderer.render(world,fluidState,pos, vOffset,this);
    }


    @Override
    public void setSprite(Sprite sprite) {
        quad.setSprite(sprite);
    }

    @Override
    public void eatVertex(float x, float y, float z, float u, float v) {
        quad.setX(i, x);
        quad.setY(i, y);
        quad.setZ(i, z);
        quad.setTexU(i, u);
        quad.setTexV(i, v);
        i = (i+1)%4;

    }

    @Override
    public void drawQuad(Vector3f offset, Quaternionf rot, Direction direction) {
        writeQuad(chunkModelBuilder, material, offset, quad, ModelQuadFacing.fromDirection(direction), false);
    }

    private void writeQuad(ChunkModelBuilder builder, Material material, Vector3f offset, ModelQuadView quad, ModelQuadFacing facing, boolean flip) {
        var vertices = this.vertices;

        for (int i = 0; i < 4; i++) {
            var out = vertices[flip ? 3 - i : i];
            out.x = offset.x + quad.getX(i);
            out.y = offset.y + quad.getY(i);
            out.z = offset.z + quad.getZ(i);
            out.color = 0xFFFFFFFF;
            out.u = quad.getTexU(i);
            out.v = quad.getTexV(i);
            out.light = 16;
        }

        Sprite sprite = quad.getSprite();

        if (sprite != null) {
            builder.addSprite(sprite);
        }

        var vertexBuffer = builder.getVertexBuffer(facing);
        vertexBuffer.push(vertices, material);
    }



}
