package quimufu.colourful_portals.config;

import com.google.common.collect.Lists;
import eu.midnightdust.lib.config.MidnightConfig;
import net.minecraft.registry.RegistryKey;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.DyeColor;
import net.minecraft.world.World;
import quimufu.colourful_portals.ColourfulPortalsMod;
import quimufu.colourful_portals.util.Procedure;

import java.util.*;

public class ColourfulPortalConfig extends MidnightConfig {

    @Entry(category = "integrations", name = "Disable Immersive Portals integration")
    public static boolean disableImmersivePortals = false;

    @Comment(category = "portal_colours")
    public static Comment explanation1;
    @Comment(category = "portal_colours")
    public static Comment explanation2;

    @Entry(category = "portal_colours")
    public static List<String> white = Lists.newArrayList("minecraft:white_wool");

    @Entry(category = "portal_colours")
    public static List<String> orange = Lists.newArrayList("minecraft:orange_wool");

    @Entry(category = "portal_colours")
    public static List<String> magenta = Lists.newArrayList("minecraft:magenta_wool");

    @Entry(category = "portal_colours")
    public static List<String> light_blue = Lists.newArrayList("minecraft:light_blue_wool");

    @Entry(category = "portal_colours")
    public static List<String> yellow = Lists.newArrayList("minecraft:yellow_wool");

    @Entry(category = "portal_colours")
    public static List<String> lime = Lists.newArrayList("minecraft:lime_wool");

    @Entry(category = "portal_colours")
    public static List<String> pink = Lists.newArrayList("minecraft:pink_wool");

    @Entry(category = "portal_colours")
    public static List<String> gray = Lists.newArrayList("minecraft:gray_wool");

    @Entry(category = "portal_colours")
    public static List<String> light_gray = Lists.newArrayList("minecraft:light_gray_wool");

    @Entry(category = "portal_colours")
    public static List<String> cyan = Lists.newArrayList("minecraft:cyan_wool");

    @Entry(category = "portal_colours")
    public static List<String> purple = Lists.newArrayList("minecraft:purple_wool");

    @Entry(category = "portal_colours")
    public static List<String> blue = Lists.newArrayList("minecraft:blue_wool");

    @Entry(category = "portal_colours")
    public static List<String> brown = Lists.newArrayList("minecraft:brown_wool");

    @Entry(category = "portal_colours")
    public static List<String> green = Lists.newArrayList("minecraft:green_wool");

    @Entry(category = "portal_colours")
    public static List<String> red = Lists.newArrayList("minecraft:red_wool");

    @Entry(category = "portal_colours")
    public static List<String> black = Lists.newArrayList("minecraft:black_wool");

    @Entry(category = "portal_colours")
    public static List<String> none = Lists.newArrayList();

    @Entry(category = "visuals")
    public static boolean blockyPortalFluid = false;


    @Entry(category = "colourful_pearl")
    public static Map<String, Integer> pearlDimensionWeights = new HashMap<>(
            Map.of("minecraft:overworld", 79,
                    "minecraft:the_nether", 20,
                    "minecraft:the_end", 1)
    );

    @Entry(category = "colourful_pearl", isSlider = true, min = 0f, max = 1f, precision = 1000)
    public static double pearlSameDimensionLikelihood = 0.9D;

    @Entry(category = "colourful_pearl", isSlider = true, min = 1 << 4, max = 1 << 16, precision = 10)
    public static double maxPearlDistance = 1 << 14;

    @Entry(category = "colourful_pearl", isSlider = true, min = 1 << 4, max = 1 << 16, precision = 10)
    public static double minPearlDistance = 1 << 12;

    private static final ArrayList<Procedure> onWrite = new ArrayList<>(2);


    public static Set<String> getAllPortalBlocks() {
        Set<String> allBlocks = new HashSet<>();
        allBlocks.addAll(white);
        allBlocks.addAll(orange);
        allBlocks.addAll(magenta);
        allBlocks.addAll(light_blue);
        allBlocks.addAll(yellow);
        allBlocks.addAll(lime);
        allBlocks.addAll(pink);
        allBlocks.addAll(gray);
        allBlocks.addAll(light_gray);
        allBlocks.addAll(cyan);
        allBlocks.addAll(purple);
        allBlocks.addAll(blue);
        allBlocks.addAll(brown);
        allBlocks.addAll(green);
        allBlocks.addAll(red);
        allBlocks.addAll(black);
        allBlocks.addAll(none);
        return allBlocks;
    }

    public static DyeColor colorOf(String block) {
        if (white.contains(block)) {
            return DyeColor.WHITE;
        } else if (orange.contains(block)) {
            return DyeColor.ORANGE;
        } else if (magenta.contains(block)) {
            return DyeColor.MAGENTA;
        } else if (light_blue.contains(block)) {
            return DyeColor.LIGHT_BLUE;
        } else if (yellow.contains(block)) {
            return DyeColor.YELLOW;
        } else if (lime.contains(block)) {
            return DyeColor.LIME;
        } else if (pink.contains(block)) {
            return DyeColor.PINK;
        } else if (gray.contains(block)) {
            return DyeColor.GRAY;
        } else if (light_gray.contains(block)) {
            return DyeColor.LIGHT_GRAY;
        } else if (cyan.contains(block)) {
            return DyeColor.CYAN;
        } else if (purple.contains(block)) {
            return DyeColor.PURPLE;
        } else if (blue.contains(block)) {
            return DyeColor.BLUE;
        } else if (brown.contains(block)) {
            return DyeColor.BROWN;
        } else if (green.contains(block)) {
            return DyeColor.GREEN;
        } else if (red.contains(block)) {
            return DyeColor.RED;
        } else if (black.contains(block)) {
            return DyeColor.BLACK;
        } else if (none.contains(block)) {
            return null; // Fully transparent portal
        } else {
            throw new IllegalArgumentException("Invalid portal block: " + block);
        }
    }

    @Override
    public void writeChanges(String modid) {
        super.writeChanges(modid);
        for (Procedure procedure : onWrite) {
            procedure.run();
        }

    }

    public static void registerListener(Procedure p) {
        onWrite.add(p);
    }

    public static void addMissingDimensionsToConfig(MinecraftServer server) {
        Set<RegistryKey<World>> worlds = server.getWorldRegistryKeys();
        boolean changed = false;
        for (RegistryKey<World> world : worlds) {
            if (!pearlDimensionWeights.containsKey(world.getValue().toString())) {
                pearlDimensionWeights.put(world.getValue().toString(), 1);
                changed = true;
            }
        }
        if (maxPearlDistance < minPearlDistance) {
            double min = maxPearlDistance;
            maxPearlDistance = minPearlDistance;
            minPearlDistance = min;
            changed = true;
        }
        if (changed) {
            write(ColourfulPortalsMod.MOD_ID);
        }
    }
}
