package quimufu.colourful_portals.general_util;

import org.jetbrains.annotations.Nullable;

public class Node<E> {

    private Node<E> next = null;
    private Node<E> previous = null;
    private E value;

    public LinkedList<E> getPartOf() {
        return partOf;
    }

    private final LinkedList<E> partOf;
    private boolean orphaned = false;

    protected Node(E value, LinkedList<E> partOf) {
        this.value = value;
        this.partOf = partOf;
    }

    public Node<E> getNext() {
        return next;
    }

    public void setNext(@Nullable Node<E> next) {
        if (next != null) {
            next.previous = this;
        }
        if(next == this){
            throw new IllegalArgumentException("cycles are not allowed");
        }
        this.next = next;
    }

    public E getValue() {
        return value;
    }

    public void setValue(E value) {
        this.value = value;
    }

    public Node<E> getPrevious() {
        return previous;
    }

    public void setPrevious(@Nullable Node<E> previous) {
        if (previous != null) {
            previous.next = this;
        }
        this.previous = previous;
    }

    public void remove() {
        if (partOf.headNode == this) {
            if (next != null) {
                partOf.headNode = next;
            } else {
                partOf.headNode = previous;
            }

        }
        if (previous != null) {
            previous.setNext(next);
        }
        if (next != null) {
            next.setPrevious(previous);
        }
        orphaned = true;
    }

    public void insertBefore(E element) {
        Node<E> newNode = new Node<>(element, partOf);
        newNode.setPrevious(this.previous);
        newNode.setNext(this);
    }

    public void insertAfter(E element) {
        Node<E> newNode = new Node<>(element, partOf);
        newNode.setNext(this.next);
        newNode.setPrevious(this);
    }

    public boolean orphaned() {
        return orphaned;
    }
}
