package quimufu.colourful_portals.mixin;

import net.minecraft.block.BlockState;
import net.minecraft.entity.Entity;
import net.minecraft.registry.Registries;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.server.world.ServerWorld;
import net.minecraft.util.Identifier;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Direction;
import net.minecraft.util.math.Vec3d;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;
import quimufu.colourful_portals.util.RaycastHelper;

import java.util.List;

import static quimufu.colourful_portals.ColourfulPortalsMod.*;

@Mixin(ServerWorld.class)
public abstract class BlockChangeAndEntityMovementMixin {
    @Unique
    private static final List<Direction> HORIZONTAL = Direction.stream().filter(d -> d.getAxis().isHorizontal()).toList();

//    @Unique
//    private final ThreadLocal<Map<EntityType<?>, Entity>> entities =
//            ThreadLocal.withInitial(() -> HashMap.newHashMap(Registries.ENTITY_TYPE.size()));

    @Shadow
    public abstract String toString();

//    @Inject(at = @At(value = "HEAD"), method = "tickEntity")
//    private void cpm_beforeEntityTick(Entity entity, CallbackInfo ci) {
//        if(PORTAL_MANAGER.getLinkingSystem().needsMovementCallback()){
//            Entity copy = entities.get().computeIfAbsent(entity.getType(), t -> t.create(null));
//            copy.copyFrom(entity);
//        }
//    }

    @Shadow
    public abstract boolean isChunkLoaded(long chunkPos);

    @Inject(at = @At("RETURN"), method = "tickEntity")
    private void cpm_afterEntityTick(Entity entity, CallbackInfo ci) {
        if (PORTAL_MANAGER.getLinkingSystem().needsMovementCallback()) {
            Vec3d prevEntityPos;
            if (entity instanceof ServerPlayerEntity serverPlayerEntity) {
                ServerPlayNetworkHandlerAccessor spnha = (ServerPlayNetworkHandlerAccessor) serverPlayerEntity.networkHandler;
                prevEntityPos = new Vec3d(spnha.getLastTickX(), spnha.getLastTickY(), spnha.getLastTickZ());
            } else {
                prevEntityPos = new Vec3d(entity.prevX, entity.prevY, entity.prevZ);
            }
            Vec3d entityPos = entity.getPos();
            ServerWorld world = (ServerWorld) (Object) this;
            List<BlockPos> passedBlocks = RaycastHelper.passedBlocks(prevEntityPos, entityPos);
            for (BlockPos passedBlock : passedBlocks) {
                if (PORTAL_MANAGER.onMovementThroughBlock(entity, passedBlock, world)) {
                    return;
                }
            }
        }
    }

    @Inject(at = @At("RETURN"), method = "onBlockChanged")
    private void cpm_afterOnBlockChanged(BlockPos pos, BlockState oldBlock, BlockState newBlock, CallbackInfo info) {
        ServerWorld world = (ServerWorld) (Object) this;
        if (world.isDebugWorld())
            return;

        // This code is injected into the end of ServerWorld.onBlockChanged()V
        if (oldBlock.getBlock() != newBlock.getBlock()) {
            if (PORTAL_BLOCKS.contains(Registries.BLOCK.getId(newBlock.getBlock()))) {
                LOGGER.debug("onBlockNew {} -> {}", oldBlock, newBlock);
                Identifier blockId = Registries.BLOCK.getId(newBlock.getBlock());
                synchronized (PORTAL_MANAGER.pendingUpdates){
                    PORTAL_MANAGER.pendingUpdates.add(() ->
                            PORTAL_MANAGER.onPortalBlockPlaced(world, pos, blockId));
                }
                if(!PORTAL_MANAGER.blockUpdatesPending.getAndSet(true)){
                    world.getServer().execute(PORTAL_MANAGER::runPendingUpdates);
                }
            }

            if (PORTAL_BLOCKS.contains(Registries.BLOCK.getId(oldBlock.getBlock()))) {
                LOGGER.debug("onBlockOld {} -> {}", oldBlock, newBlock);
                Identifier blockId = Registries.BLOCK.getId(oldBlock.getBlock());
                synchronized (PORTAL_MANAGER.pendingUpdates){
                    PORTAL_MANAGER.pendingUpdates.add(() ->
                            PORTAL_MANAGER.onPortalBlockBroken(world, pos, blockId));
                }
                if(!PORTAL_MANAGER.blockUpdatesPending.getAndSet(true)){
                    world.getServer().execute(PORTAL_MANAGER::runPendingUpdates);
                }
            }
        }
}

}